;;/////////////////////////////////////////////////////////////////////////////
;;Plotting points from x-y coordinates using the PICTURE object of Open Music
;;Background texture of the PICTURE object is defined with an ascii code figure
;;/////////////////////////////////////////////////////////////////////////////
;;Mauricio Rodriguez
;;2SI Spring/2010
;;/////////////////////////////////////////////////////////////////////////////

;;====================
;;Drawing coordinates:
;;====================

(in-package :om)

(defun plot-point (x y color)
  (make-instance 'graphic-object 
                 :graph-type 'pen 
                 :graph-points (list (list x y)) 
                 :graph-color (om-make-color (first color) (second color) (third color)) 
                 :graph-size 2
                 :graph-line 'line 
                 :graph-fill t
                 :graph-contents nil))


(defun plotting-points (x-list y-list color)
  (loop for x in x-list
        for y in y-list
        collect (plot-point x y color)))


(defun plotting-squares (points-list color)
  (make-instance 'graphic-object 
                 :graph-type 'rect 
                 :graph-points (group-list points-list 2 'linear) 
                 :graph-color (om-make-color (first color) (second color) (third color)) 
                 :graph-size 1
                 :graph-line 'line 
                 :graph-fill t
                 :graph-contents nil))


(defun color-case (color)
  (case color
    ('red '(1 0 0))
    ('green '(0 1 0))  
    ('yellow '(1 1 0))
    ('purple '(0.75 0.60 0.95))))


;;====================
;;Get-texture:
;;====================

(defun choose-file-dialog (&key (prompt "Open file") (operation :open))
  (let ((screen (capi:convert-to-screen)))
    (capi:screen-width screen)
    (capi:screen-height screen)
    (capi:prompt-for-file prompt 
                          :operation operation 
                          :owner screen 
                          :filters '("All Files" "*.*"))))


(defvar *new-data* nil)


#|

;;Parse to identity

(defun load-file ()
  (setf *new-data* nil)
  (setf *new-data*
        (with-open-file (input (choose-file-dialog)
                               :direction 
                               :input)
          (when input
            (loop for line = (read-line input nil)
                  while line do (format t "~a~%" line))))))


;;(load-file)
;;*new-data*

|#


(defun load-file* ()
  (setf *new-data* nil)
  (setf *new-data*
        (with-open-file (input (choose-file-dialog)
                               :direction 
                               :input)
          (read-from input))))

(defun read-from (stream)
  (let ((current-line (read-line stream nil)))
    (unless (not current-line)
      (cons current-line
            (read-from stream)))))

;;(load-file*)
;;*new-data*

#|

(defun load-file ()
  (load-file*)
  (loop for x in *new-data*
        collect (read-from-string (concatenate 'string "(" x ")"))))

;;(load-file)

|#


(defun load-file ()
  (load-file*)
  (setf *new-data*
        (loop for x in *new-data*
              append (read-from-string (concatenate 'string "(" x ")")))))

;;(load-file)

(defun expand-symbol (symbol-sequence)
  (read-from-string 
   (remove #\# 
           (remove #\\ 
                   (write-to-string 
                    (coerce 
                     (write-to-string symbol-sequence) 
                     'list))))))

;;(expand-symbol '**--------**)

(defun symbol-to-color (symbol-list color-1 color-2)
  (if (endp symbol-list)
    nil
    (cons (cond ((equal (first symbol-list) '*) color-1)
                ((equal (first symbol-list) '-) color-2))
          (symbol-to-color (rest symbol-list) color-1 color-2))))

;;(symbol-to-color (expand-symbol '**--------**) '(1.0 0.0 0.0) '(0.0 0.0 1.0))

(defun render-texture (color* color-)
  (load-file)
  (loop for x in *new-data*
        collect (symbol-to-color (expand-symbol x) color* color-)))
        

;;(render-texture '(1.0 0.0 0.0) '(1.0 1.0 0.0))

#|

Load a file such as the following ascii text:

************
**--------**
**--------**
**--------**
**--------**
************

|#

